#!/usr/bin/perl
#
# diam_microbe_ident2.csv(微生物情報 - 分離源、至適生育温度等 (生理学的情報）)
#
#

package csv2rdf;

use Text::CSV_XS;
use Text::CSV;
use IO::File;
use utf8;

binmode STDOUT, ':utf8';
# 
$taxurl ="http://identifiers.org/taxonomy/";

#--------------------------------------------------------------------------------
# diam_microbe_ident2.csvのTurtle処理
# (微生物情報 - 分離源、至適生育温度等 (生理学的情報）)
#--------------------------------------------------------------------------------
sub make_rdf_ident2_main {

        my $fh = $_[0];
	my $filename = $_[1];

        # RDF Header
        my $header = '@prefix xml: <http://www.w3.org/XML/1998/namespace>'.".\n".'@prefix xsd: <http://www.w3.org/2001/XMLSchema#>'.".\n".'@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>'.".\n".'@prefix mccv: <http://purl.jp/bio/01/mccv#>'.".\n".'@prefix prism: <http://prismstandard.org/namespaces/basic/2.0/>'.".\n";

        # header出力
        printf $header."\n";

        # ファイル読み込み
	my $csv = Text::CSV->new ({ binary => 1, eol => $/ });
	open my $io, "<", $filename or die "$filename: $!";
	#my $file = IO::File->new($filename, "r");

	$line_cnt = 0;
        #while (not $file->eof and my $items = $csv->getline($file)) {	
	while (my $items = $csv->getline ($io)) {
		if($line_cnt == 0) {
			$line_cnt++;
			next;
		}

                # カラム単位で処理
                make_rdf_ident2_str($items);

		$line_cnt++;
        }
}

sub make_rdf_ident2_str {

	# ブランクノード
	printf "[]\n";

        my $items = $_[0];

	$species = $items->[0];
	$taxid = $items->[1];
	$strain = $items->[2];
	$isolation = $items->[3];
	$temperature = $items->[4];
	$literture = $items->[6];


	# 生物種
	printf("mccv:MCCV_000011 \"%s\";\n", $species);
	# 分離源
	if("$isolation" ne "" && "$isolation" ne '""') {
		printf("mccv:MCCV_000030 \"%s\"\@ja;\n", "$isolation");
	}
	# TaxonomyId
	printf("mccv:MCCV_000065 <%s%s>;\n", $taxurl, $taxid);

	# 生育温度
	print_temp($temperature);

	# 出典
	if(index("${literture}",'"') != -1 ) {
		printf("prism:originPlatform %s", "${literture}");
		printf("\n");
		printf(";\n");
	} else {
		printf("prism:originPlatform \"%s", "${literture}");
		printf("\n");
		printf("\";\n");
	}

	printf("a mccv:MCCV_000002\n");

	printf ".\n\n";

}

# 生育温度出力
sub print_temp {
	my $temp = $_[0];

	# カラムが空の場合はreturn
	if($temp eq "") {
		return;
	}

	# (,),-を半角化
	$temp =~ s/（/\(/g;
	$temp =~ s/）/\)/g;

	# MCCVを定義
	$mccv14 = "";
	$mccv15 = "";
	$mccv16 = "";
	$mccv69 = "";
	$mccv70 = "";

	# 「()」が存在する場合
	if($temp =~ /\(/) {
		# ()内と外を分割
		$in_val = $`;
		$out_val = $';

		# ()内の値に「-」が存在する場合
		if($in_val =~ /-/) {
			$mccv69 = $`;
			$mccv70 = $';
			$mccv70 =~ s/\)//g;
			
			printf("mccv:MCCV_000069 \"%s\"^^xsd:decimal;\n", $mccv69);
			printf("mccv:MCCV_000070 \"%s\"^^xsd:decimal;\n", $mccv70);
		} else {
			# 「(」より前の文字列を取得
			$mccv69 = $`;
			$mccv69 =~ s/\)//g;
			printf("mccv:MCCV_000069 \"%s\"^^xsd:decimal;\n", $mccv69);
		}

		# ()外の値に「-」が存在する場合
                if($out_val =~ /-/) {
                        $mccv15 = $`;
                        $mccv16 = $';
			$mccv16 =~ s/\)//g;

                        printf("mccv:MCCV_000015 \"%s\"^^xsd:decimal;\n", $mccv15);
                        printf("mccv:MCCV_000016 \"%s\"^^xsd:decimal;\n", $mccv16);
                } else {
                        # 「(」より前の文字列を取得
                        $mccv14 = $out_val;
                        printf("mccv:MCCV_000014 \"%s\"^^xsd:decimal;\n", $mccv14);
                }

	} else {
                # 値に「-」が存在する場合
                if($temp =~ /-/) {
                        $mccv15 = $`;
                        $mccv16 = $';
                        $mccv16 =~ s/\)//g;

                        printf("mccv:MCCV_000015 \"%s\"^^xsd:decimal;\n", $mccv15);
                        printf("mccv:MCCV_000016 \"%s\"^^xsd:decimal;\n", $mccv16);
                } else {
                        # 「(」より前の文字列を取得
                        $mccv14 = $temp;
                        printf("mccv:MCCV_000014 \"%s\"^^xsd:decimal;\n", $mccv14);
                }

	}

}

1;
