#!/usr/bin/perl
#
# diam_microbe_ident8(キノン組成 (Bacteriaの化学成分情報))
#
#

package csv2rdf;

use Text::CSV_XS;
use Text::CSV;
use IO::File;
use File::Basename;
use utf8;

binmode STDOUT, ':utf8';

# require設定
require "diam_microbe_ident4.pl";

# TaxsonomyId
$taxurl ="http://identifiers.org/taxonomy/";

#--------------------------------------------------------------------------------
# diam_microbe_ident8.csvのTurtle処理
#--------------------------------------------------------------------------------
sub make_rdf_ident8_main {

	# ログopen
	open(LOG, ">:utf8", "../logs/ident8.log") or die("error :$!");

        my $fh = $_[0];
	my $filename = $_[1];

        # RDF Header
        my $header = '@prefix xml: <http://www.w3.org/XML/1998/namespace>'.".\n".'@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>'.".\n".'@prefix mccv: <http://purl.jp/bio/01/mccv#>'.".\n".'@prefix mpo: <http://purl.jp/bio/01/mpo#>'.".\n".'@prefix prism: <http://prismstandard.org/namespaces/basic/2.0/>'.".\n";

        # header出力
        printf $header."\n";

        # ファイル読み込み
	my $csv = Text::CSV->new ({ binary => 1, eol => $/ });
	open my $io, "<", $filename or die "$filename: $!";
	#my $file = IO::File->new($filename, "r");

	$line_cnt = 0;
        #while (not $file->eof and my $items = $csv->getline($file)) {	
	while (my $items = $csv->getline ($io)) {
		if($line_cnt == 0) {
			$line_cnt++;
			next;
		}

                # カラム単位で処理
                make_rdf_ident8_str($items);

		$line_cnt++;
#exit;
        }
	
	# ログclose
	close(LOG);
}


#--------------------------------------------------------------------------------
# diam_microbe_ident8.csvのカラムごとの出力処理
#--------------------------------------------------------------------------------
sub make_rdf_ident8_str {

	# ブランクノード
	printf "[]\n";

        my $items = $_[0];

	# カラム取得
	$species = $items->[0];
        $taxid = $items->[1];
        $strains = $items->[2];

	# キノン配列を初期化
	@q_arry =();
	set_quinones($items->[3]);
	set_quinones($items->[4]);
	set_quinones($items->[5]);

        # 生物種
        printf("mccv:MCCV_000011 \"%s\";\n", $species);

        # OtherStrainNumber
        # 「=」で分割
        @strain = split(/[=\/,]/, $strains);
        for($i = 0;$i < @strain;$i++ ){
                print_osn(@strain[$i]);
        }

	# キノン
	print_quinones();

        # TaxonomyId
        printf("mccv:MCCV_000065 <%s%s>;\n", $taxurl, $taxid);

	printf("a mccv:MCCV_000002\n");

	printf ".\n\n";

}

# キノン生成を出力
sub print_quinones {
	$cnt = @q_arry;

	for($i=0;$i < $cnt;$i++) {
#printf("#########   @q_arry[$i]:\n");
		$str = `xsltproc  --stringparam label "@q_arry[$i]" ../bin/xslt/quinones.xsl ../data/mpo.rdf`;
		# XML宣言、改行除去
		$str =~ s/<\?xml version="1.0"\?>//g;
		$str =~ s/\n//g;
        	
		# キノン生成
		if($str ne "") {
			# 成形
			$str = basename($str);
			$str =~ s/#/:/;
			printf("mpo:MPO_10017 %s;\n", $str);
		} else {
			error_log($line_cnt+1,@q_arry[$i]);
		}
	}
}

sub set_quinones {
	$quinoes = $_[0];
	if($quinoes eq "") {
		return;
	}
	# 「(」が存在する場合
	if($quinoes =~ /\(/) {
		# 「()」内に「,」が存在するか
		$out_val =$`;
		$in_val = $';
		if($in_val =~ /,/) {
			# ex. MK-9(H2,H4)
			@q_tmp = split(/,/ ,$in_val);
			$len = @q_tmp;
			for($i = 0;$i < $len;$i++ ){
				$str1 = @q_tmp[$i];
				$str1 =~ s/[()]//g;
				$str1 = $out_val."(".$str1.")";
				set_array($str1);
			}
		} else {
			# 「()」外に「,」が存在するか
			if($out_val =~ /,/) {
				# ex. MK-9,10(H4)

				@q_tmp = split(/[, ]/ ,$out_val);
				$len = @q_tmp;
				for($i = 0;$i < $len;$i++ ){
					set_array(@q_tmp[$i]);
				}
				set_array($in_val);
			} else {
				# ex. MK-9(H4)
				set_array($quinoes);
			}
		} 
	} else {
		# 「,」が存在する場合
		if($quinoes =~ /,/) {
			# 「,」で配列に分割
			@q_tmp = split(/[, ]/ ,$quinoes);
			@q_tmp = reset_array(@q_tmp);
			$len = @q_tmp;

			# prefixの生成
			if($len > 1 && substr(@q_tmp[0],0,1) ne substr(@q_tmp[1],0,1)) {
				$prefix = substr(@q_tmp[0],0, index(@q_tmp[0],"-")+1);
			}
			for ($i = 0;$i < $len;$i++){
				if($i == 0) {
					set_array(@q_tmp[$i]);
				} else {
					set_array($prefix.@q_tmp[$i]);
				}
			}
		} else {
			set_array($quinoes);
		}
	}

}

sub set_array {
	# 半角ブランクで分割
	@tmp = split(/ / ,$_[0]);
	$length= @tmp;

	for($j =0; $j < $length;$j++){
		# キノン配列に挿入
		if(@tmp[$j] ne "") {
			push(@q_arry, @tmp[$j]);
		}
	}
}

sub reset_array {
	my @array = @_;
	my @r_array;

	foreach my $val(@array) {
		$val =~ s/ //g;
		if($val ne "") {
			push(@r_array, $val);
		}
	}
	return @r_array;
}

sub error_log {
	$line = $_[0];
	$val = $_[1];

	$mes ="diam_microbe_ident8.csvの${line}行目のキノン組成が取得できませんでした。 : $val\n";
	print LOG "$mes";
}

1;
