<?php
require_once('NIESWebPage.class.php');
require_once('NIESUtil.php');

class NIES {
	public $subject;
	public $otherCollectionNumbers;
	public $name;
	public $organismType;
	public $historyOfDeposit;
	public $dateOfIsolation;
	public $isolatedFrom;
	public $isolatedFromString;
	public $geographicOrigin;
	public $geographicOriginString;
	public $medium1;
	public $medium2;
	public $literature;
	
	private $webPage;
	
	public static $PREFIX = array(
		"gaz:"  => "http://purl.obolibrary.org/obo/",
		"mccv:" => "http://purl.jp/bio/01/mccv#",
		"meo:" => "http://purl.jp/bio/11/meo/",
		"prism:" => "http://prismstandard.org/namespaces/1.2/basic/",
	);
	
	public function toRDF(){
		
		// TODO RDF
	}
	
	public function toString(){
		return $this->subject . "\t" . 
		$this->otherCollectionNumbers . "\t" . 
		$this->name . "\t" . 
		$this->organismType . "\t" . 
		$this->historyOfDeposit . "\t" . 
		$this->dateOfIsolation . "\t" . 
		$this->isolatedFrom . "\t" . 
		$this->isolatedFromString . "\t" . 
		$this->geographicOrigin . "\t" . 
		implode(',', $this->geographicOriginString) . "\t" . 
		$this->medium1 . "\t" . 
		$this->medium2 . "\t" . 
		$this->literature;
	}
	
	/** 設定 */
	public function setFromCSVArray($csv){
		// Webページの情報取得
		$this->webPage = new NIESWebPage($csv[0]);
		
		$this->setSubject($csv[0]);
		$this->setOtherCollectionNumbers($csv[19]);
		$this->setName($csv[3]);
		$this->setOrganismType($csv[2]);
		$this->setHistoryOfDeposit($csv[9]);
		$this->setDateOfIsolation($csv[10]);
		$this->setIsolatedFrom($csv[8]);
		$this->setIsolatedFromString($csv[8]);
		$this->setGeographicOrigin($csv[7]);
		$this->setGeographicOriginString($csv[7]);
		$this->setMedium1($csv[14]);
		$this->setMedium2($csv[14]);
		$this->setLiterature($csv[0]);
	}
	
	private function setSubject($str){
		$this->subject = $this->webPage->getDetailURL();
	}
	private function setOtherCollectionNumbers($str){
		$this->otherCollectionNumbers = $str;
	}
	private function setName($str){
		$this->name = $str;
	}
	private function setOrganismType($str){
		//OrganismTypeにコンバート
		$this->organismType = getOrganismType($str);
	}
	private function setHistoryOfDeposit($str){
		$this->historyOfDeposit = $str;
	}
	private function setDateOfIsolation($str){
		$this->dateOfIsolation = $str;
	}
	private function setIsolatedFrom($str){
		// ()外の文字列はMEOのマッチングを行い、合致したClassのrdf:aboutを取得する。※4
		if(preg_match('/^([^\\(]*) ?\\((.+)\\)$/', $str, $matches)){
			$this->isolatedFrom = getHabitat(trim($matches[1]));
			return;
		}
		$this->isolatedFrom = null;
	}
	private function setIsolatedFromString($str){
		// ()内の文字列をLiteralで出力。
		if(preg_match('/\\((.+)\\)/', $str, $matches)){
			$this->isolatedFromString = $matches[1];
			return;
		}
		$this->isolatedFromString = null;
	}
	private function setGeographicOrigin($str){
		// 先頭から半角ブランクまでを抽出し、Gazettiaのnameとマッチングする。合致した場合、Gazettiaのidを取得する。
		$list = explode(" ",$str);
		$result = getGazzetter(array($list[0]));
		$this->geographicOrigin = count($result) ? $result[0] : null;
	}
	private function setGeographicOriginString($str){
		// 先頭の半角ブランク以降、「()までを抽出し、Gazettiaのnameとマッチングする。合致した場合、Gazettiaのidを取得する。Gazettiaに存在しない場合、Literalとして登録する。
		if(preg_match('/^[^ ]+ (.*) ?\\(.+\\)/',$str,$matches)){
			// まずリテラル成分を入れる
			$this->geographicOriginString[] = $matches[1];
			
			// 文字列を分割してGazettiaに聞く
			$list = preg_split('/[, ]+/', $matches[1]);
			$this->geographicOriginString = array_merge($this->geographicOriginString, getGazzetter($list));
			return;
		}
		
		$this->geographicOriginString = array();
	}
	private function setMedium1($str){
		// セミコロン「;」で分割した配列のうち2番目の要素を抽出し、「Temperature:」以降を取得する。
		if(preg_match('/^ *Temperature:([^;]*)$/',$str,$matches)){
			$this->medium1 = trim($matches[1]);
			return;
		}
		$this->medium1 = null;
	}
	private function setMedium2($str){
		// セミコロン「;」で分割した配列のうち1番目の要素を抽出し、「Medium:」以降を取得する。
		$list = explode(";", $str);
		if(!empty($list[0])){
			if(preg_match('/^Medium:(.*)$/',$list[0],$matches)){
				$this->medium2 = trim($matches[1]);
				return;
			}
		}
		$this->medium2 = null;
	}
	private function setLiterature($str){
		$this->literature = $this->webPage->getReference();
	}
}
?>