<?php
require_once('simple_html_dom.php');
class NIESWebPage {
	
	/** リクエストベースURI */
	const REQUEST_BASE_URI = "http://mcc.nies.go.jp";
	
	/** NIES ID */
	private $niesId;
	
	/** StainId */
	private $strainId;
	
	function __construct($niesId){
		$this->niesId = $niesId;
	}
	
	/** 株番号取得 */
	public function getStrainNumber(){
		return str_replace('NIES-','',$this->niesId);
	}
	
	/** 詳細ページのURL取得 */
	function getDetailURL(){
		$html = file_get_html($this->getSearchURL());
		foreach($html->find('div.strainlist table tr td a') as $e){
			$link = $e->href;
			$link = html_entity_decode($link);	// デコード
			$link = preg_replace('/;jsessionid=[^\?]+/i','',$link);	// セッションが付いていることがあるので削除
			$link = self::REQUEST_BASE_URI . $link;
			
			// StainIDを求める
			$this->strainId = $this->parseStrainId($link);
			
			return $link;
		}
	}
	
	/** StrainIdをパースして取得 */
	private function parseStrainId($str){
			$parsed_url = parse_url($str);
			
			// クエリが存在し、stainId=という引数が存在したらそのあとを取得
			if(!empty($parsed_url['query'])){
				$parsed_url['query'] = explode("&",$parsed_url['query']);
				foreach($parsed_url['query'] as $val){
					if(preg_match('/strainId=(\\d+)/',$val,$matches)) return $matches[1];
				}
			}
			else{
				return null;
			}

	}
	
	/** 検索URIを取得 */
	public function getSearchURL(){
		return sprintf(self::REQUEST_BASE_URI . "/numberSearch.do?strainNumber=%d", $this->getStrainNumber());
	}
	
	/** 検索URIを取得 */
	public function getReferenceURL(){
		return sprintf(self::REQUEST_BASE_URI . "/referenceSearch.do?strainId=%d", $this->getStrainId());
	}
	
	/** StrainIDを取得 */
	public function getStrainId(){
		return $this->strainId;
	}
	
	/** ReferenceページのURL取得 */
	public function getReference(){
		$html = file_get_html($this->getReferenceURL());
		$list = $html->find('div.content form[name=referenceListPrintForm] table tr td');
		
		// リストの2番目を取って空白を削除し、実体参照を呼び出し。
		$e = empty($list[1]) ? "" : $list[1]->plaintext;
		$e = mb_convert_encoding($e, "UTF-8","SJIS-win, UTF-8");
		$e = preg_replace('/[ \t]+/',' ',$e);
		$e = html_entity_decode(str_replace('&nbsp;',' ',$e));
		$e = trim($e);
		
		return $e;
	}
}
